@@ -5,7 +5,7 @@ require 'net/http'
 require 'tempfile'
 require 'uri'
 require 'pathname'
-require 'json'
+require 'puppet/util/json'
 require 'semantic_puppet'
 
 class Puppet::Forge < SemanticPuppet::Dependency::Source
@@ -17,6 +17,9 @@ class Puppet::Forge < SemanticPuppet::Dependency::Source
 
   USER_AGENT = "PMT/1.1.1 (v3; Net::HTTP)".freeze
 
+  # From https://forgeapi.puppet.com/#!/release/getReleases
+  MODULE_RELEASE_EXCLUSIONS=%w[readme changelog license uri module tags supported file_size downloads created_at updated_at deleted_at].join(',').freeze
+
   attr_reader :host, :repository
 
   def initialize(host = Puppet[:module_repository], strict_semver = true)
@@ -56,7 +59,7 @@ class Puppet::Forge < SemanticPuppet::Dependency::Source
     matches = []
     uri = "/v3/modules?query=#{term}"
     if Puppet[:module_groups]
-      uri += "&module_groups=#{Puppet[:module_groups].gsub('+', ' ')}"
+      uri += "&module_groups=#{Puppet[:module_groups].tr('+', ' ')}"
     end
 
     while uri
@@ -64,7 +67,7 @@ class Puppet::Forge < SemanticPuppet::Dependency::Source
       response = make_http_request(uri)
 
       if response.code == '200'
-        result = JSON.parse(response.body)
+        result = Puppet::Util::Json.load(response.body)
         uri = decode_uri(result['pagination']['next'])
         matches.concat result['results']
       else
@@ -90,9 +93,9 @@ class Puppet::Forge < SemanticPuppet::Dependency::Source
   # @see SemanticPuppet::Dependency::Source#fetch
   def fetch(input)
     name = input.tr('/', '-')
-    uri = "/v3/releases?module=#{name}&sort_by=version"
+    uri = "/v3/releases?module=#{name}&sort_by=version&exclude_fields=#{MODULE_RELEASE_EXCLUSIONS}"
     if Puppet[:module_groups]
-      uri += "&module_groups=#{Puppet[:module_groups].gsub('+', ' ')}"
+      uri += "&module_groups=#{Puppet[:module_groups].tr('+', ' ')}"
     end
     releases = []
 
@@ -101,7 +104,7 @@ class Puppet::Forge < SemanticPuppet::Dependency::Source
       response = make_http_request(uri)
 
       if response.code == '200'
-        response = JSON.parse(response.body)
+        response = Puppet::Util::Json.load(response.body)
       else
         raise ResponseError.new(:uri => URI.parse(@host).merge(uri), :response => response)
       end
@@ -167,6 +170,8 @@ class Puppet::Forge < SemanticPuppet::Dependency::Source
     def prepare
       return @unpacked_into if @unpacked_into
 
+      Puppet.warning "#{@metadata['name']} has been deprecated by its author! View module on Puppet Forge for more info." if deprecated?
+
       download(@data['file_uri'], tmpfile)
       validate_checksum(tmpfile, @data['file_md5'])
       unpack(tmpfile, tmpdir)
@@ -210,6 +215,10 @@ class Puppet::Forge < SemanticPuppet::Dependency::Source
         raise RuntimeError, _("Could not extract contents of module archive: %{message}") % { message: e.message }
       end
     end
+
+    def deprecated?
+      @data['module'] && (@data['module']['deprecated_at'] != nil)
+    end
   end
 
   private
@@ -231,6 +240,6 @@ class Puppet::Forge < SemanticPuppet::Dependency::Source
   def decode_uri(uri)
     return if uri.nil?
 
-    URI.decode(uri.gsub('+', ' '))
+    URI.decode(uri.tr('+', ' '))
   end
 end
