@@ -224,7 +224,7 @@ class Puppet::Transaction::Report
     @external_times ||= {}
     @host = Puppet[:node_name_value]
     @time = Time.now
-    @report_format = 9
+    @report_format = 10
     @puppet_version = Puppet.version
     @configuration_version = configuration_version
     @transaction_uuid = transaction_uuid
@@ -346,7 +346,7 @@ class Puppet::Transaction::Report
     report = raw_summary
 
     ret = ""
-    report.keys.sort { |a,b| a.to_s <=> b.to_s }.each do |key|
+    report.keys.sort_by(&:to_s).each do |key|
       ret += "#{Puppet::Util::Metric.labelize(key)}:\n"
 
       report[key].keys.sort { |a,b|
@@ -459,8 +459,7 @@ class Puppet::Transaction::Report
   def calculate_time_metrics
     metrics = Hash.new(0)
     resource_statuses.each do |name, status|
-      type = Puppet::Resource.new(name).type
-      metrics[type.to_s.downcase] += status.evaluation_time if status.evaluation_time
+      metrics[status.resource_type.downcase] += status.evaluation_time if status.evaluation_time
     end
 
     @external_times.each do |name, value|
